/*
 * extension.js
 * Gnome3 Jump to Overview Extension
 *
 * Opens the Overview when the last window on the current workspace is closed (Gnome-Shell < 3.10 behaviour)
 *
 *
 * Author: Simon Junga (simonthechipmunk@gmx.de)
 *
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 */


//***// imports:

// main functionality
const Main = imports.ui.main;
const Mainloop = imports.mainloop;

const GLib = imports.gi.GLib;

// own imports
const Me = imports.misc.extensionUtils.getCurrentExtension();
const Prefs = Me.imports.prefs;



// define global variables
let workspaceChanged, currentWorkspace, windowRemoved;
let eventStartup=null;


//***// basic extension functions

function init() {
 	// initialize preferences
	Prefs.init();

}




function enable() {
        // connect to 'workspace-switched' signal
        workspaceChanged = global.screen.connect('workspace-switched', _checkWorkspace);

	// execute _checkWorkspace at extension start
	_checkWorkspace();


	// open the Overview on Shell startup if set in the preferences
	if(Prefs._getOnStartup() && global.display.focus_window == null) {

		_checkStartup();
	
		// if overview didn't show, try again after 1 second
		// (may be caused by a slow machine or autostart applications)
		if(Main.overview._shown == false){

			eventStartup = GLib.timeout_add_seconds(0, 1, _checkStartup);
		}

	}



}




function disable() {
        // disconnect from signals
        global.screen.disconnect(workspaceChanged);
        currentWorkspace.disconnect(windowRemoved);

	// remove timer event
	if(eventStartup) {
		Mainloop.source_remove(eventStartup);
	}


}




//***// extension functions

function _checkWorkspace() {

        // disconnect former 'window-removed' signal (if any)
        if(currentWorkspace) {
        currentWorkspace.disconnect(windowRemoved);
        }
   
        // get the current workspace
        currentWorkspace = global.screen.get_active_workspace();

        // connect to window-removed on current workspace
        windowRemoved = currentWorkspace.connect('window-removed', function() { 


                // get active windows on current workspace and number of workspaces
                let windowList = currentWorkspace.list_windows();
		let activeWindows = windowList.length;
                let activeWorkspaces = global.screen.get_n_workspaces();

		// ignore widgets (e.g. screenlets) and minimized windows 
		for each (var window in windowList) {
			if(window.is_skip_taskbar() || (!window.showing_on_its_workspace() 
				&& Prefs._getIgnoreMinimized()) ) {activeWindows --};
		}


                // check for "last window closed", "overview shown" and "last active workspace" if set in the preferences
                if( (activeWindows < 1 && Main.overview._shown == false && activeWorkspaces > 2 && !Prefs._getOnLastWS())
			|| (activeWindows < 1 && Main.overview._shown == false && Prefs._getOnLastWS()) ) {
                        // show Overview
                        Main.overview.show.call(Main.overview);
                }
            
	});
 

}




function _checkStartup() {

	// check if desktops are empty and show the overview (open windows cause glitches in Overview at Shell restart)
	let activeWorkspacesStartup = global.screen.get_n_workspaces();
	let checkOK = true;
	
	// check every active workspace
	for (i = 0; i < activeWorkspacesStartup; i ++) {

		let currentWorkspaceCheck = global.screen.get_workspace_by_index(i);
		let windowListCheck = currentWorkspaceCheck.list_windows();
		let activeWindowsCheck = windowListCheck.length;

		
		for each (var window in windowListCheck) {
			if(window.is_skip_taskbar() ) {activeWindowsCheck --};
		}

		if(activeWindowsCheck > 0) {
			// check returns false if open windows are found
			checkOK = false;
			break;
		}

	}



	 if (checkOK && Main.overview._shown == false) {
		// show Overview
		Main.overview.show.call(Main.overview);

	}

return false;

}
